def doc():
    print("""This python library is made to connect to Smart Prospective API, in order to manage your:
- Account
- Materials
- Buildings
- Material Groups
- Users
- Medias
- Tags
- Rights
- Right Groups
- PostAccount
- Notifications
- Bills
- Deliveries
- Interventions

Documentation & Samples: https://github.com/smart-prospective/smart-prospective-api
""")


if __name__ == "__main__":
    doc()
